@@ -7,8 +7,8 @@
  * any later version.
  *
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 510 $
+ * $Date: 2013-09-28 00:09:30 +0800 (Sat, 28 Sep 2013) $
  */
 #include "Document.hpp"
 
@@ -36,6 +36,10 @@ Document::Document(wxWindow* parent) : wxPanel(parent), res(NULL),
   //
 }
 
+Document::~Document() {
+  //
+}
+
 void Document::set(ImageResource* res, Model* model, Controller* control) {
   this->res = res;
   this->model = model;
@@ -55,11 +59,8 @@ void Document::save() {
   /* pre-condition */
   assert (havePath);
 
-  unsigned width = res->getWidth();
-  unsigned height = res->getHeight();
-
-  wxImage img(width, height);
-  model->calcFg(width, height, img);
+  wxImage img(res->getWidth(), res->getHeight());
+  model->calcFg(1, img);
   img.SetOption(wxIMAGE_OPTION_QUALITY, 95);
   wxLogNull logNo;
   isSaved = img.SaveFile(path);
