@@ -7,8 +7,8 @@
  * any later version.
  *
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 510 $
+ * $Date: 2013-09-28 00:09:30 +0800 (Sat, 28 Sep 2013) $
  */
 #ifndef INDII_GUI_DOCUMENT_HPP
 #define INDII_GUI_DOCUMENT_HPP
@@ -46,6 +46,11 @@ public:
    */
   Document(wxWindow* parent);
 
+  /**
+   * Destructor.
+   */
+  virtual ~Document();
+
   /**
    * Is image saved?
    */
