@@ -7,13 +7,14 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 509 $
+ * $Date: 2013-09-27 19:37:55 +0800 (Fri, 27 Sep 2013) $
  */
 #ifndef INDII_GUI_HELP_HPP
 #define INDII_GUI_HELP_HPP
 
 #include "wx/html/helpctrl.h"
+#include "wx/menu.h"
 
 namespace indii {  
 /**
@@ -24,7 +25,7 @@ public:
   /**
    * Initialize help system.
    */
-  static void init(wxWindow* parent);
+  static void init(wxWindow* parent = NULL);
 
   /**
    * Terminate help system.
@@ -44,15 +45,6 @@ private:
    */
   static wxHtmlHelpController* help;
   
-  /**
-   * Top-level frame for help window.
-   *
-   * @note Any of the wxHtmlHelp* classes causes a crash in Photoshop on Mac
-   * on the second invocation of the plugin. Use something generic instead
-   * and combine with wxHF_EMBEDDED
-   */
-  static wxFrame* frame;
-  
   /**
    * Was the help file successfully loaded?
    */
