@@ -7,8 +7,8 @@
  * any later version.
  *
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 515 $
+ * $Date: 2013-09-29 19:04:13 +0800 (Sun, 29 Sep 2013) $
  */
 #include "Window.hpp"
 
@@ -44,7 +44,6 @@ Window::Window(wxWindow* parent, const Mode mode, WindowObserver* obs) :
     wxDefaultSize, (mode == STANDALONE) ? INDII_STANDALONE_FLAGS :
     INDII_EMBED_FLAGS), obs(obs), exiting(false), mode(mode), manager(NULL),
     main(NULL), notebook(NULL), doc(NULL), infoBar(NULL) {
-  Help::init(this);
   main = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL|wxBORDER_NONE);
   infoBar = new InfoBar(this);
   if (mode == STANDALONE) {
@@ -60,9 +59,12 @@ Window::~Window() {
   wxConfig config(_(PACKAGE_TARNAME), _("indii.org"));
   wxString persp = manager->SavePerspective();
   config.Write(_("Perspective_v2"), persp);
+  config.Write(_("x"), GetPosition().x);
+  config.Write(_("y"), GetPosition().y);
+  config.Write(_("width"), GetSize().GetWidth());
+  config.Write(_("height"), GetSize().GetHeight());
   
   manager->UnInit();
-  Help::term();
 }
 
 void Window::openAndAdd(const wxString& file) {
@@ -166,9 +168,9 @@ void Window::OnMenuOpen(wxMenuEvent& evt) {
     mnRecent->OnShow(evt);
   } else if (evt.GetMenu() == mnView) {
     if (mode == STANDALONE && notebook->GetPageCount() > 0) {
+      #ifdef URBANLIGHTSCAPE
       Document* doc;
       doc = dynamic_cast<Document*>(notebook->GetPage(notebook->GetSelection()));
-      #ifdef URBANLIGHTSCAPE
       bool overlay = doc->getPreviewPane()->getPreviewImage()->getOverlay();
       mnView->Check(ID_BUT_OVERLAY, overlay);
       #endif
@@ -536,9 +538,9 @@ void Window::init(Document* doc) {
 
   #ifdef __WXOSX__
   /* help system causing crash under Photoshop on Cocoa, to do with menu perhaps? */
-  if (mode != EMBED) {
+  //if (mode != EMBED) {
     tbTop->AddTool(wxID_HELP, _("Help"), *_img_gnome_help_browser, _("Help"));
-  }
+  //}
   #else
   tbTop->AddTool(wxID_HELP, _("Help"), *_img_gnome_help_browser, _("Help"));
   #endif
@@ -550,7 +552,7 @@ void Window::init(Document* doc) {
     tbTop->AddTool(ID_BUT_REGISTER, _("Register"), *_img_gnome_emblem_web, _("Register"));
   }
   #ifdef __WXOSX__
-  if (mode != EMBED) {
+  //if (mode != EMBED) {
     #ifndef WX_28
     tbTop->AddStretchableSpace();
     tbTop->AddControl(new wxContextHelpButton(tbTop, wxID_CONTEXT_HELP, wxDefaultPosition,
@@ -559,7 +561,7 @@ void Window::init(Document* doc) {
     tbTop->AddSeparator();
     tbTop->AddControl(new wxContextHelpButton(tbTop));
     #endif
-  }
+  //}
   #else
   #ifndef WX_28
   tbTop->AddStretchableSpace();
@@ -580,7 +582,7 @@ void Window::init(Document* doc) {
   /* AUI manager */
   manager = new wxAuiManager(main);
 
-  int width, height;
+  int x, y, width, height;
 
   /* notebook */
   wxAuiPaneInfo paneInfo;
@@ -660,8 +662,28 @@ void Window::init(Document* doc) {
   }
 
   SetSizer(sizer);
-  SetSize(1000,640);
   SetReturnCode(wxID_CANCEL);
+
+  long val;
+  width = intround(0.8f*wxSystemSettings::GetMetric(wxSYS_SCREEN_X));
+  height = intround(0.8f*wxSystemSettings::GetMetric(wxSYS_SCREEN_Y));
+  if (config.Read(_("width"), &val)) {
+    width = static_cast<int>(val);
+  }
+  if (config.Read(_("height"), &val)) {
+    height = static_cast<int>(val);
+  }
+  SetSize(wxSize(width, height));
+  CentreOnScreen();
+  x = GetPosition().x;
+  y = GetPosition().y;
+  if (config.Read(_("x"), &val)) {
+    x = static_cast<int>(val);
+  }
+  if (config.Read(_("y"), &val)) {
+    y = static_cast<int>(val);
+  }
+  SetPosition(wxPoint(x, y));
   manager->Update();
   
   if (doc != NULL) {
