@@ -7,27 +7,28 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 506 $
+ * $Date: 2013-09-25 21:48:47 +0800 (Wed, 25 Sep 2013) $
  */
 #include "FileImageResource.hpp"
 
+#include "ImageManipulation.hpp"
+
+#include "wx/image.h"
+
 using namespace indii;
 
 FileImageResource::FileImageResource(const wxString& path) {
-  /* original image */
-  original = new wxImage();
-  if (!original->LoadFile(path)) {
+  wxImage img;
+  if (img.LoadFile(path)) {
+    original = new Image(img.GetWidth(), img.GetHeight());
+    convert(img, *original);
+    cache = new ImageCache(original);
+  } else {
     throw 1;
   }
-  
-  /* caches */
-  lowCache = new ImageCache(original, false);
-  highCache = new ImageCache(original, true);
 }
 
 FileImageResource::~FileImageResource() {
-  delete lowCache;
-  delete highCache;  
-  delete original;
+  //
 }
