@@ -7,12 +7,14 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 511 $
+ * $Date: 2013-09-28 15:50:18 +0800 (Sat, 28 Sep 2013) $
  */
 #ifndef INDII_IMAGE_IMAGECACHE_HPP
 #define INDII_IMAGE_IMAGECACHE_HPP
 
+#include "Image.hpp"
+
 #include "wx/image.h"
 
 #include <map>
@@ -26,10 +28,9 @@ public:
   /**
    * Constructor.
    *
-   * @param image Original image.
-   * @param high Should images be resized in high quality?
+   * @param original Original image.
    */
-  ImageCache(wxImage* image, const bool high = false);
+  ImageCache(Image* original);
       
   /**
    * Destructor.
@@ -39,46 +40,49 @@ public:
   /**
    * Get resized image.
    *
-   * @param width Width of the image.
-   * @param height Height of the image.
+   * @param scale Inverse scale of the new image.
    *
    * @return The resized image, drawn from the cache if available.
-   *
-   * If either @p width or @p height is <= 0, the original image is
-   * returned.
    */
-  wxImage* get(const int width = 0, const int height = 0);
+  Image* get(const int scale);
+
+  /**
+   * Get display image.
+   *
+   * @param width Width of the image in pixels.
+   */
+  wxImage* getDisplay(const int width);
 
 private:
-  /**
-   * Image dimension.
-   */
-  typedef std::pair<const int, const int> dim_t;
-  
   /**
    * Image cache.
    */
-  typedef std::map<dim_t,wxImage*> cache_t;
-  
+  typedef std::map<int,Image*> cache_t;
+
   /**
-   * Image cache pair.
+   * Display image cache.
    */
-  typedef std::pair<dim_t,wxImage*> cache_pair_t;
-  
+  typedef std::map<int,wxImage*> display_cache_t;
+
   /**
    * Original image.
    */
-  wxImage* original;
+  Image* original;
   
+  /**
+   * Original display image.
+   */
+  wxImage* displayOriginal;
+
   /**
    * Cache.
    */
   cache_t cache;
 
   /**
-   * Should images be resized in high quality?
+   * Display cache.
    */
-  const bool high;
+  display_cache_t displayCache;
 };
 }
 
