@@ -7,8 +7,8 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 517 $
+ * $Date: 2013-09-30 23:33:29 +0800 (Mon, 30 Sep 2013) $
  */
 #include "ImageManipulation.hpp"
 
@@ -60,6 +60,34 @@ void indii::show(wxImage& image, const wxRect& rect) {
   }
 }
 
+#ifndef __PLUGIN__
+void indii::convert(const Image& in, wxImage& out) {
+  /* pre-condition */
+  assert(in.getWidth() == out.GetWidth() && in.getHeight() == out.GetHeight());
+
+  int x, y;
+  for (y = 0; y < in.getHeight(); ++y) {
+    for (x = 0; x < in.getWidth(); ++x) {
+      out.SetRGB(wxRect(x, y, 1, 1), uround(in.r(y, x)), uround(in.g(y, x)), uround(in.b(y, x)));
+    }
+  }
+}
+#endif
+
+void indii::convert(const wxImage& in, Image& out) {
+  /* pre-condition */
+  assert(in.GetWidth() == out.getWidth() && in.GetHeight() == out.getHeight());
+
+  int x, y;
+  for (x = 0; x < in.GetWidth(); ++x) {
+    for (y = 0; y < in.GetHeight(); ++y) {
+      out.r(y, x) = in.GetRed(x, y)/255.0f;
+      out.g(y, x) = in.GetGreen(x, y)/255.0f;
+      out.b(y, x) = in.GetBlue(x, y)/255.0f;
+    }
+  }
+}
+
 wxString indii::wildcards(const bool save) {
   wxList handlers = wxImage::GetHandlers();
 
