@@ -7,8 +7,8 @@
  * any later version.
  *
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 510 $
+ * $Date: 2013-09-28 00:09:30 +0800 (Sat, 28 Sep 2013) $
  */
 #ifndef INDII_MODEL_CONTROLLER_HPP
 #define INDII_MODEL_CONTROLLER_HPP
@@ -26,6 +26,11 @@ public:
    */
   Controller();
   
+  /**
+   * Destructor.
+   */
+  virtual ~Controller();
+
   /**
    * Set overlay on or off.
    */
@@ -96,14 +101,6 @@ protected:
 };
 }
 
-inline indii::Controller::Controller() : overlay(true) {
-  //
-}
-
-inline void indii::Controller::showOverlay(const bool on) {
-  this->overlay = on;
-}
-
 inline void indii::Controller::up(const int x0, const int y0, const int x,
     const int y, wxMouseEvent& evt) {
   evt.Skip();
