@@ -7,14 +7,14 @@
  * any later version.
  *
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 503 $
+ * $Date: 2013-09-23 00:17:02 +0800 (Mon, 23 Sep 2013) $
  */
 #include "Model.hpp"
 
 using namespace indii;
 
-Model::Model() {
+Model::Model(ImageResource* res) : res(res) {
   this->greyR = 76.0f / 255.0f;
   this->greyG = 150.0f / 255.0f;
   this->greyB = 29.0f / 255.0f;
