@@ -7,13 +7,15 @@
  * any later version.
  *
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 500 $
- * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
+ * $Rev: 506 $
+ * $Date: 2013-09-25 21:48:47 +0800 (Wed, 25 Sep 2013) $
  */
 #ifndef INDII_MODEL_MODEL_HPP
 #define INDII_MODEL_MODEL_HPP
 
 #include "Observable.hpp"
+#include "../image/ImageManipulation.hpp"
+#include "../image/ImageResource.hpp"
 #include "../image/ColourSpace.hpp"
 
 #include "wx/image.h"
@@ -23,12 +25,17 @@ namespace indii {
 /**
  * Abstract model.
  */
-class Model : public Observable {
+class Model: public Observable {
 public:
   /**
    * Constructor.
    */
-  Model();
+  Model(ImageResource* res);
+
+  /**
+   * Get underlying image resource.
+   */
+  ImageResource* getImageResource();
 
   /**
    * Get red channel proportion for conversion to greyscale.
@@ -95,46 +102,73 @@ public:
    * Calculate foreground of subimage.
    *
    * @param rect Rectangular region of interest, relative to scaled image.
-   * @param width Scaled width of image.
-   * @param height Scaled height of image.
+   * @param scale Inverse scale of image.
    * @param[out] o Output image.
    */
-  virtual void calcFg(const wxRect& rect, const unsigned width,
-      const unsigned height, wxImage& o);
+  virtual void calcFg(const wxRect& rect, const int scale, Image& o);
 
   /**
    * Calculate foreground of whole image.
    *
-   * @param width Scaled width of image.
-   * @param height Scaled height of image.
+   * @param scale Inverse scale of image.
    * @param[out] o Output image.
    */
-  virtual void calcFg(const unsigned width, const unsigned height,
-      wxImage& o);
+  virtual void calcFg(const int scale, Image& o);
+
+  /**
+   * Calculate foreground of image.
+   *
+   * @param[out] o Output image.
+   */
+  virtual void calcFg(Image& o);
+
+  /**
+   * Calculate foreground of subimage.
+   *
+   * @param rect Rectangular region of interest, relative to scaled image.
+   * @param scale Inverse scale of image.
+   * @param[out] o Output image.
+   */
+  virtual void calcFg(const wxRect& rect, const int scale, wxImage& o);
+
+  /**
+   * Calculate foreground of whole image.
+   *
+   * @param scale Inverse scale of image.
+   * @param[out] o Output image.
+   */
+  virtual void calcFg(const int scale, wxImage& o);
+
+  /**
+   * Calculate foreground of image.
+   *
+   * @param[out] o Output image.
+   */
+  virtual void calcFg(wxImage& o);
 
   /**
    * Calculate annotation layer of subimage.
    *
    * @param rect Rectangular region of interest, relative to scaled image.
-   * @param width Scaled width of image.
-   * @param height Scaled height of image.
-   * @param zoom Zoom factor to create this scale.
+   * @param scale Inverse scale of image.
    * @param[out] dc Device context to annotate.
    */
-  virtual void calcAn(const wxRect& rect, const unsigned width,
-      const unsigned height, wxDC& dc);
+  virtual void calcAn(const wxRect& rect, const int scale, wxDC& dc);
 
   /**
    * Calculate annotation layer of whole image.
    *
-   * @param width Scaled width of image.
-   * @param height Scaled height of image.
+   * @param scale Inverse scale of image.
    * @param[out] dc Device context to annotate.
    */
-  virtual void calcAn(const unsigned width, const unsigned height,
-      wxDC& dc);
+  virtual void calcAn(const int scale, wxDC& dc);
 
 protected:
+  /**
+   * Image resource.
+   */
+  ImageResource* res;
+
   /**
    * Colour space model.
    */
@@ -157,6 +191,10 @@ protected:
 };
 }
 
+inline indii::ImageResource* indii::Model::getImageResource() {
+  return res;
+}
+
 inline float indii::Model::getRedMix() const {
   return greyR;
 }
@@ -169,26 +207,51 @@ inline float indii::Model::getBlueMix() const {
   return greyB;
 }
 
-inline void indii::Model::calcFg(const wxRect& rect,
-    const unsigned width, const unsigned height, wxImage& o) {
+inline void indii::Model::calcFg(const wxRect& rect, const int scale,
+    Image& o) {
   //
 }
 
-inline void indii::Model::calcFg(const unsigned width,
-    const unsigned height, wxImage& o) {
-  wxRect rect(0, 0, width, height);
-  calcFg(rect, width, height, o);
+inline void indii::Model::calcFg(const int scale, Image& o) {
+  /* pre-condition */
+  assert(o.getWidth() == res->getWidth()/scale);
+  assert(o.getHeight() == res->getHeight()/scale);
+
+  wxRect rect(0, 0, o.getWidth(), o.getHeight());
+  calcFg(rect, scale, o);
 }
 
-inline void indii::Model::calcAn(const wxRect& rect,
-    const unsigned width, const unsigned height, wxDC& dc) {
+inline void indii::Model::calcFg(Image& o) {
+  calcFg(1, o);
+}
+
+inline void indii::Model::calcFg(const wxRect& rect, const int scale,
+    wxImage& o) {
+  Image img(o.GetWidth(), o.GetHeight());
+  calcFg(rect, scale, img);
+  convert(img, o);
+}
+
+inline void indii::Model::calcFg(const int scale, wxImage& o) {
+  Image img(o.GetWidth(), o.GetHeight());
+  calcFg(scale, img);
+  convert(img, o);
+}
+
+inline void indii::Model::calcFg(wxImage& o) {
+  Image img(o.GetWidth(), o.GetHeight());
+  calcFg(img);
+  convert(img, o);
+}
+
+inline void indii::Model::calcAn(const wxRect& rect, const int scale,
+    wxDC& dc) {
   //
 }
 
-inline void indii::Model::calcAn(const unsigned width,
-    const unsigned height, wxDC& dc) {
-  wxRect rect(0, 0, width, height);
-  calcAn(rect, width, height, dc);
+inline void indii::Model::calcAn(const int scale, wxDC& dc) {
+  wxRect rect(0, 0, res->getWidth()/scale, res->getHeight()/scale);
+  calcAn(rect, scale, dc);
 }
 
 #endif
