@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2007-2012 Lawrence Murray
+ * Copyright (C) 2007-2013 Lawrence Murray
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -7,8 +7,8 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 438 $
- * $Date: 2012-03-16 23:02:51 +0800 (Fri, 16 Mar 2012) $
+ * $Rev: 500 $
+ * $Date: 2013-08-16 19:04:13 +0800 (Fri, 16 Aug 2013) $
  */
 #include "ThumbPane.hpp"
 
@@ -43,8 +43,8 @@ ThumbPane::ThumbPane(wxWindow* parent) : wxScrolledWindow(parent),
   //butAdd->SetLabel(wxEmptyString);
   //butSub->SetLabel(wxEmptyString);
   #else
-  butAdd = new wxBitmapButton(this, ID_BUT_ADD, *_img_gnome_list_add);
-  butSub = new wxBitmapButton(this, ID_BUT_SUB, *_img_gnome_list_remove);
+  butAdd = new wxBitmapButton(this, ID_BUT_ADD, *_img_gnome_list_add, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW|wxBORDER_NONE);
+  butSub = new wxBitmapButton(this, ID_BUT_SUB, *_img_gnome_list_remove, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW|wxBORDER_NONE);
   #endif
 
   wxGridBagSizer* buttonSizer = new wxGridBagSizer(0,0);
