@@ -7,8 +7,8 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 502 $
- * $Date: 2013-08-17 12:38:57 +0800 (Sat, 17 Aug 2013) $
+ * $Rev: 518 $
+ * $Date: 2013-10-03 20:43:12 +0800 (Thu, 03 Oct 2013) $
  */
 #include "ClusterModel.hpp"
 
@@ -26,23 +26,22 @@ namespace ublas = boost::numeric::ublas;
 using namespace indii;
 using namespace std;
 
-ClusterModel::ClusterModel(ImageResource* res) : minClusterer(NULL),
-    locked(false) {
-  this->res = res;
-  this->saturationThreshold = 16;
-  this->maxPixels = 10000;
-  this->reps = 8;
+ClusterModel::ClusterModel(ImageResource* res) :
+    Model(res), minClusterer(NULL), locked(false) {
+  this->saturationThreshold = 0.0625f;
+  this->maxPixels = 50000;
+  this->reps = 12;
   this->k = 4;
-  this->hard = 1.0f;
-  this->saturationDecay = 16.0f / 255.0f;
+  this->hard = 0.9f;
+  this->saturationDecay = 0.0625f;
   this->centroidDecay = 0.0f;
-  this->saturationSoftness = 32.0f / 255.0f;
+  this->saturationSoftness = 0.125f;
   this->centroidSoftness = 0.0f;
   this->colours.resize(k, *wxWHITE);
   this->hues.resize(k, 0.0f);
   this->sats.resize(k, -1.0f);
   this->lights.resize(k, 0.0f);
-  this->alphas.resize(k, 255.0f);
+  this->alphas.resize(k, 1.0f);
   this->minClusterer = new KMeansClusterer<>(k, time(NULL));
 }
 
@@ -52,16 +51,16 @@ ClusterModel::~ClusterModel() {
 
 void ClusterModel::setDefaults() {
   lock();
-  setHard(1.0f);
-  setSaturationThreshold(16);
-  setMaxPixels(10000);
-  setNumRepetitions(8);
+  setHard(0.9f);
+  setSaturationThreshold(0.0625f);
+  setMaxPixels(50000);
+  setNumRepetitions(12);
   setNumClusters(4);
-  setSaturationDecay(16.0f / 255.0f);
+  setSaturationDecay(0.0625f);
   setCentroidDecay(0.0f);
-  setSaturationSoftness(32.0f / 255.0f);
+  setSaturationSoftness(0.125f);
   setCentroidSoftness(0.0f);
-  setChannelMix(76.0f, 150.0f, 29.0f);
+  setChannelMix(0.299f, 0.587f, 0.114f);
   setForDialog();
   unlock();
   prepare();
@@ -75,7 +74,7 @@ void ClusterModel::setForDialog() {
   fill(hues.begin(), hues.end(), 0.0f);
   fill(sats.begin(), sats.end(), -1.0f);
   fill(lights.begin(), lights.end(), 0.0f);
-  fill(alphas.begin(), alphas.end(), 255.0f);
+  fill(alphas.begin(), alphas.end(), 1.0f);
 }
 
 void ClusterModel::setNumClusters(const unsigned k) {
@@ -89,7 +88,7 @@ void ClusterModel::setNumClusters(const unsigned k) {
     fill(sats.begin(), sats.end(), -1.0f);
     fill(hues.begin(), hues.end(), 0.0f);
     fill(lights.begin(), lights.end(), 0.0f);
-    fill(alphas.begin(), alphas.end(), 255.0f);
+    fill(alphas.begin(), alphas.end(), 1.0f);
 
     delete minClusterer;
     minClusterer = new KMeansClusterer<>(k, time(NULL));
@@ -101,8 +100,8 @@ void ClusterModel::setNumClusters(const unsigned k) {
 
 void ClusterModel::setHard(const float hard) {
   /* pre-condition */
-  assert (hard >= 0.0f && hard <= 1.0f);
-  
+  assert(hard >= 0.0f && hard <= 1.0f);
+
   if (this->hard != hard) {
     this->hard = hard;
     notifyAll();
@@ -118,7 +117,7 @@ void ClusterModel::setNumRepetitions(const unsigned reps) {
   }
 }
 
-void ClusterModel::setSaturationThreshold(const unsigned char x) {
+void ClusterModel::setSaturationThreshold(const float x) {
   if (this->saturationThreshold != x) {
     this->saturationThreshold = x;
     prepare();
@@ -138,7 +137,7 @@ void ClusterModel::setMaxPixels(const unsigned n) {
 
 void ClusterModel::setSaturationDecay(const float x) {
   /* pre-condition */
-  assert (x >= 0.0f && x <= 1.0f);
+  assert(x >= 0.0f && x <= 1.0f);
 
   if (this->saturationDecay != x) {
     this->saturationDecay = x;
@@ -149,7 +148,7 @@ void ClusterModel::setSaturationDecay(const float x) {
 
 void ClusterModel::setCentroidDecay(const float x) {
   /* pre-condition */
-  assert (x >= 0.0f && x <= 1.0f);
+  assert(x >= 0.0f && x <= 1.0f);
 
   if (this->centroidDecay != x) {
     this->centroidDecay = x;
@@ -160,7 +159,7 @@ void ClusterModel::setCentroidDecay(const float x) {
 
 void ClusterModel::setSaturationSoftness(const float x) {
   /* pre-condition */
-  assert (x >= 0.0 && x <= 1.0f);
+  assert(x >= 0.0 && x <= 1.0f);
 
   if (this->saturationSoftness != x) {
     this->saturationSoftness = x;
@@ -171,7 +170,7 @@ void ClusterModel::setSaturationSoftness(const float x) {
 
 void ClusterModel::setCentroidSoftness(const float x) {
   /* pre-condition */
-  assert (x >= 0.0 && x <= 1.0f);
+  assert(x >= 0.0 && x <= 1.0f);
 
   if (this->centroidSoftness != x) {
     this->centroidSoftness = x;
@@ -182,12 +181,12 @@ void ClusterModel::setCentroidSoftness(const float x) {
 
 void ClusterModel::setColour(const unsigned i, const wxColour& col) {
   /* pre-condition */
-  assert (i < k);
+  assert(i < k);
 
   ClusterVector<>::type x(ClusterVector<>::N);
-  x(0) = (float)col.Red();
-  x(1) = (float)col.Green();
-  x(2) = (float)col.Blue();
+  x(0) = col.Red()/255.0f;
+  x(1) = col.Green()/255.0f;
+  x(2) = col.Blue()/255.0f;
   indii::KMeansClusterer<>::distance_type::prepare(x);
 
   colours[i] = col;
@@ -196,8 +195,8 @@ void ClusterModel::setColour(const unsigned i, const wxColour& col) {
 
 void ClusterModel::setHue(const unsigned i, const float x) {
   /* pre-condition */
-  assert (i < k);
-  assert (x >= -0.5f && x <= 0.5f);
+  assert(i < k);
+  assert(x >= -0.5f && x <= 0.5f);
 
   if (hues[i] != x) {
     hues[i] = x;
@@ -208,8 +207,8 @@ void ClusterModel::setHue(const unsigned i, const float x) {
 
 void ClusterModel::setSat(const unsigned i, const float x) {
   /* pre-condition */
-  assert (i < k);
-  assert (x >= -1.0f && x <= 1.0f);
+  assert(i < k);
+  assert(x >= -1.0f && x <= 1.0f);
 
   if (sats[i] != x) {
     sats[i] = x;
@@ -220,8 +219,8 @@ void ClusterModel::setSat(const unsigned i, const float x) {
 
 void ClusterModel::setLight(const unsigned i, const float x) {
   /* pre-condition */
-  assert (i < k);
-  assert (x >= -255.0f && x <= 255.0f);
+  assert(i < k);
+  assert(x >= -1.0f && x <= 1.0f);
 
   if (lights[i] != x) {
     lights[i] = x;
@@ -234,8 +233,8 @@ void ClusterModel::setLight(const unsigned i, const float x) {
 
 void ClusterModel::setAlpha(const unsigned i, const float x) {
   /* pre-condition */
-  assert (i < k);
-  assert (x >= 0.0f && x <= 255.0f);
+  assert(i < k);
+  assert(x >= 0.0f && x <= 1.0f);
 
   if (alphas[i] != x) {
     alphas[i] = x;
@@ -248,7 +247,7 @@ void ClusterModel::setAlpha(const unsigned i, const float x) {
 
 void ClusterModel::show(const unsigned i, const bool on) {
   /* pre-condition */
-  assert (i < k);
+  assert(i < k);
 
   if (isShown(i) != on) {
     if (on) {
@@ -256,7 +255,7 @@ void ClusterModel::show(const unsigned i, const bool on) {
     } else {
       sats[i] = -1.0f;
     }
-    
+
     notifyAll();
     notifySatChange(i);
   }
@@ -269,120 +268,102 @@ void ClusterModel::showAll(const bool on) {
   }
 }
 
-void ClusterModel::calcFg(const wxRect& rect,
-    const unsigned width, const unsigned height, wxImage& o) {
+void ClusterModel::calcFg(const wxRect& rect, const int scale, Image& o) {
   /* pre-conditions */
-  assert (o.GetWidth() >= rect.width);
-  assert (o.GetHeight() >= rect.height);
+  assert(o.getWidth() == rect.width);
+  assert(o.getHeight() == rect.height);
 
-  wxImage img(res->calc(rect, width, height, true));
-  if (!o.HasAlpha()) {
-    o.InitAlpha();
-  }
+  Image& img = *res->get(scale);
 
-  #pragma omp parallel default(shared)
+  #pragma omp parallel
   {
     ColourSpace::rgb_t rgb(3);
     ColourSpace::hsl_t hsl(3);
-    ClusterVector<>::type pixel(ClusterVector<>::N);
     std::vector<float> ds(k);
 
-    int x, y;
-    int cluster;
-    unsigned char r, g, b/*, a*/;
+    int x, y, cluster;
     float decay, light, sat;
 
     #pragma omp for schedule(dynamic)
-    for (y = 0; y < rect.height; y++) {
-      for (x = 0; x < rect.width; x++) {
-        r = img.GetRed(x, y);
-        g = img.GetGreen(x, y);
-        b = img.GetBlue(x, y);
-
-        rgb(0) = r;
-        rgb(1) = g;
-        rgb(2) = b;
+    for (y = 0; y < rect.height; ++y) {
+      for (x = 0; x < rect.width; ++x) {
+        rgb(0) = img.r(rect.y + y, rect.x + x);
+        rgb(1) = img.g(rect.y + y, rect.x + x);
+        rgb(2) = img.b(rect.y + y, rect.x + x);
         cs.rgb2hsl(rgb, hsl);
-
-        pixel(0) = r;
-        pixel(1) = g;
-        pixel(2) = b;
-
         decay = threshold(hsl(1), saturationDecay, saturationSoftness);
-        
+
         /* cluster adjustments */
-        indii::KMeansClusterer<>::distance_type::prepare(pixel);
+        indii::KMeansClusterer<>::distance_type::prepare(rgb);
         if (isHard()) {
-          cluster = minClusterer->assign(pixel);
+          cluster = minClusterer->assign(rgb);
 
           /* hue */
-          hsl(0) = fmod(6.0f + hsl(0) + 6.0f*hues[cluster], 6.0f);
+          hsl(0) = fmod(1.0f + hsl(0) + hues[cluster], 1.0f);
 
           /* saturation */
-          sat = decay*sats[cluster];
+          sat = decay * sats[cluster];
           if (sat >= 0.0f) {
-            hsl(1) *= 1.0f/(1.0f - sat);
+            hsl(1) /= 1.0f - sat;
           } else {
-            hsl(1) *= (1.0f + sat)/1.0f;
+            hsl(1) *= 1.0f + sat;
           }
-          hsl(1) = cs.bound(0.0f, decay*hsl(1), 1.0f);
-          //hsl(1) = cs.bound(0.0f, decay*(sats[cluster] + 1.0f)*hsl(1), 1.0f);
+          hsl(1) = bound(0.0f, decay * hsl(1), 1.0f);
+          //hsl(1) = bound(0.0f, decay*(sats[cluster] + 1.0f)*hsl(1), 1.0f);
 
           /* lightness */
           light = lights[cluster];
           if (light >= 0.0f) {
-            hsl(2) *= 255.0f/(255.0f - light);
+            hsl(2) /= 1.0f - light;
           } else {
-            hsl(2) *= (255.0f + light)/255.0f;
+            hsl(2) *= 1.0f + light;
           }
-          hsl(2) = cs.bound(0.0f, hsl(2), 255.0f);
+          hsl(2) = bound(0.0f, hsl(2), 1.0f);
 
           /* alpha */
-          //a = cs.bound(0.0f, decay*alphas[cluster], 255.0f);
+          //a = bound(0.0f, decay*alphas[cluster], 255.0f);
         } else {
-          minClusterer->distances(pixel, ds);
-          
+          minClusterer->distances(rgb, ds);
+
           unsigned i;
-          float h = 0.0f, s = 0.0f, l = 0.0f, a = 0.0f;
-          float D = 0.0f;
+          float h = 0.0f, s = 0.0f, l = 0.0f/*, a = 0.0f*/, D = 0.0f;
           for (i = 0; i < k; ++i) {
-            ds[i] = std::pow(1.0f - ds[i], 256.0f*hard);
+            ds[i] = std::pow(1.0f - ds[i], 256.0f * hard);
             D += ds[i];
           }
-
           if (D >= 0.0f) {
             for (i = 0; i < k; ++i) {
               ds[i] /= D;
               if (ds[i] > 0.0f && ds[i] <= 1.0f) { // D >= 0.0 check may use extended precision, so ds[i] may not be finite
-                h += ds[i]*hues[i];
-                s += ds[i]*sats[i];
-                l += ds[i]*lights[i];
+                h += ds[i] * hues[i];
+                s += ds[i] * sats[i];
+                l += ds[i] * lights[i];
               }
             }
           }
           //a += ds[i]*alphas[i];
-          
+
           /* hue */
-          hsl(0) = fmod(6.0f + hsl(0) + 6.0f*h, 6.0f);
+          hsl(0) = fmod(1.0f + hsl(0) + h, 1.0f);
 
           /* saturation */
-          sat = decay*s;
+          sat = decay * s;
           if (sat >= 0.0f) {
-            hsl(1) *= 1.0f/(1.0f - sat);
+            hsl(1) /= 1.0f - sat;
           } else {
-            hsl(1) *= (1.0f + sat)/1.0f;
+            hsl(1) *= 1.0f + sat;
           }
-          hsl(1) = cs.bound(0.0f, decay*hsl(1), 1.0f);
-          //hsl(1) = cs.bound(0.0f, decay*(s + 1.0f)*hsl(1), 1.0f);
+          hsl(1) = bound(0.0f, decay * hsl(1), 1.0f);
+          //hsl(1) = bound(0.0f, decay*(s + 1.0f)*hsl(1), 1.0f);
 
           /* lightness */
           light = l;
           if (light >= 0.0f) {
-            hsl(2) *= 255.0f/(255.0f - light);
+            hsl(2) /= 1.0f - light;
           } else {
-            hsl(2) *= (255.0f + light)/255.0f;
+            hsl(2) *= 1.0f + light;
           }
-          hsl(2) = cs.bound(0.0f, hsl(2), 255.0f);
+          hsl(2) = bound(0.0f, hsl(2), 1.0f);
 
           /* alpha */
           // already set
@@ -390,97 +371,84 @@ void ClusterModel::calcFg(const wxRect& rect,
 
         /* convert back */
         cs.hsl2rgb(hsl, rgb);
-        o.SetRGB(x, y, rgb(0), rgb(1), rgb(2));
-        //o.SetAlpha(x, y, a);
+        o.r(y, x) = rgb(0);
+        o.g(y, x) = rgb(1);
+        o.b(y, x) = rgb(2);
+        //o.a(y, x) = a;
       }
     }
   }
 }
 
-void ClusterModel::calcFg(const unsigned width, const unsigned height,
-    wxImage& o) {
-  wxRect rect(0, 0, width, height);
-	calcFg(rect, width, height, o);
-}
-
-void ClusterModel::calcAlpha(const unsigned i,
-    const wxRect& rect, const unsigned width, const unsigned height,
-    channel& c) {
+void ClusterModel::calcAlpha(const unsigned i, const wxRect& rect,
+    const int scale, channel& c) {
   /* pre-conditions */
-  assert (i < k);
-  assert ((int)c.size1() == rect.height && (int)c.size2() == rect.width);
+  assert(i < k);
+  assert((int )c.size1() == rect.height && (int )c.size2() == rect.width);
 
-  wxImage img(res->calc(rect, width, height));
+  Image* img = res->get(scale);
 
-  #pragma omp parallel default(shared)
+#pragma omp parallel default(shared)
   {
     ColourSpace::rgb_t rgb(3);
     ColourSpace::hsl_t hsl(3);
     ClusterVector<>::type pixel(ClusterVector<>::N);
 
     int x, y;
-    unsigned char r, g, b;
 
-    #pragma omp for schedule(dynamic)
-    for (y = 0; y < rect.height; y++) {
-      for (x = 0; x < rect.width; x++) {
-        r = img.GetRed(x, y);
-        g = img.GetGreen(x, y);
-        b = img.GetBlue(x, y);
-
-        rgb(0) = r;
-        rgb(1) = g;
-        rgb(2) = b;
+#pragma omp for schedule(dynamic)
+    for (y = 0; y < rect.height; ++y) {
+      for (x = 0; x < rect.width; ++x) {
+        rgb(0) = img->r(rect.y + y, rect.x + x);
+        rgb(1) = img->g(rect.y + y, rect.x + x);
+        rgb(2) = img->b(rect.y + y, rect.x + x);
         cs.rgb2hsl(rgb, hsl);
+        pixel = rgb;
 
-        pixel(0) = r;
-        pixel(1) = g;
-        pixel(2) = b;
-        
         /* cluster adjustments */
         indii::KMeansClusterer<>::distance_type::prepare(pixel);
         if (minClusterer->assign(pixel) == i) {
-          c(y,x) = cs.uround(255.0f*threshold(hsl(1), saturationDecay, saturationSoftness));
+          c(y, x) = uround(
+              255.0f * threshold(hsl(1), saturationDecay, saturationSoftness));
         } else {
-          c(y,x) = 0;
+          c(y, x) = 0;
         }
       }
     }
   }
 }
 
-sparse_mask ClusterModel::calcMask(const unsigned i,
-    const wxRect& rect, const unsigned width, const unsigned height) {
+sparse_mask ClusterModel::calcMask(const unsigned i, const wxRect& rect,
+    const int scale) {
   /* pre-condition */
-  assert (i < k);
+  assert(i < k);
 
   sparse_mask m(rect.height, rect.width);
-  wxImage img(res->calc(rect, width, height));
+  Image* img = res->get(scale);
 
-  #pragma omp parallel default(shared)
+#pragma omp parallel default(shared)
   {
+    ColourSpace::rgb_t rgb(3);
+    ColourSpace::hsl_t hsl(3);
     ClusterVector<>::type pixel(ClusterVector<>::N);
-    unsigned char r, g, b;
     int x, y;
 
-    #pragma omp for schedule(dynamic)
-    for (y = 0; y < rect.height; y++) {
-      for (x = 0; x < rect.width; x++) {
-        r = img.GetRed(x,y);
-        g = img.GetGreen(x,y);
-        b = img.GetBlue(x,y);
-
-        pixel(0) = static_cast<float>(r);
-        pixel(1) = static_cast<float>(g);
-        pixel(2) = static_cast<float>(b);
+#pragma omp for schedule(dynamic)
+    for (y = 0; y < rect.height; ++y) {
+      for (x = 0; x < rect.width; ++x) {
+        rgb(0) = img->r(rect.y + y, rect.x + x);
+        rgb(1) = img->g(rect.y + y, rect.x + x);
+        rgb(2) = img->b(rect.y + y, rect.x + x);
+        cs.rgb2hsl(rgb, hsl);
+        pixel = rgb;
 
         indii::KMeansClusterer<>::distance_type::prepare(pixel);
         if (minClusterer->assign(pixel) == i) {
-          #pragma omp critical
+#pragma omp critical
           {
             /* sparse matrix, so need to control access, elements may be
              * rearranged */
-            m(y,x) = true;
+            m(y, x) = true;
           }
         }
       }
@@ -495,50 +463,43 @@ void ClusterModel::prepare() {
   }
 
   /* prepare data set for clustering */
-  wxImage* original = res->get();
-  wxImage* working;
-  int originalWidth = original->GetWidth();
-  int originalHeight = original->GetHeight();
-  int pixels = originalWidth*originalHeight;
-  int workingWidth, workingHeight;
-  float factor = maxPixels / static_cast<float>(pixels);
-    
-  if (factor > 1.0) {
+  Image* original = res->get();
+  Image* working;
+  int originalWidth = res->getWidth();
+  int originalHeight = res->getHeight();
+  int pixels = originalWidth * originalHeight;
+  int scale = (pixels + maxPixels - 1) / maxPixels;
+
+  if (scale <= 1) {
     /* image size doesn't exceed limit, cluster it directly */
-    workingWidth = originalWidth;
-    workingHeight = originalHeight;
-    working = original;
+    working = new Image(*original, 1);
   } else {
     /* image size exceeds limit, cluster scaled version */
-    workingWidth = static_cast<int>(originalWidth*sqrt(factor));
-    workingHeight = static_cast<int>(originalHeight*sqrt(factor));
-    working = res->get(workingWidth, workingHeight);
+    working = new Image(*original, scale);
   }
+  int workingWidth = working->getWidth();
+  int workingHeight = working->getHeight();
 
-  data.clear();
-  
   /* prepare data set for clustering */
-  #pragma omp parallel default(shared)
+  data.clear();
+#pragma omp parallel
   {
     ClusterVector<>::type pixel(ClusterVector<>::N);
     int x, y;
-    unsigned char r, g, b, s;
-    
-    for (x = 0; x < workingWidth; x++) {
-      #pragma omp for schedule(dynamic,16)
-      for (y = 0; y < workingHeight; y++) {
-        r = working->GetRed(x,y);
-        g = working->GetGreen(x,y);
-        b = working->GetBlue(x,y);
-        
+    float s;
+
+#pragma omp for
+    for (x = 0; x < workingWidth; ++x) {
+      for (y = 0; y < workingHeight; ++y) {
+        pixel(0) = working->r(y, x);
+        pixel(1) = working->g(y, x);
+        pixel(2) = working->b(y, x);
+
         /* saturation threshold */
-        s = saturation(r,g,b);
+        s = saturation(pixel(0), pixel(1), pixel(2));
         if (s >= saturationThreshold) {
-          pixel(0) = r;
-          pixel(1) = g;
-          pixel(2) = b;
           indii::KMeansClusterer<>::distance_type::prepare(pixel);
-          #pragma omp critical
+#pragma omp critical
           {
             data.add(pixel);
           }
@@ -546,6 +507,7 @@ void ClusterModel::prepare() {
       }
     }
   }
+  delete working;
 }
 
 void ClusterModel::cluster() {
@@ -556,20 +518,20 @@ void ClusterModel::cluster() {
   const unsigned seed = time(NULL);
   float minError = std::numeric_limits<float>::max();
 
-  #pragma omp parallel shared(minError)
+#pragma omp parallel shared(minError)
   {
     /* cluster */
     KMeansClusterer<>* clusterer;
     float error;
     int i;
     ClusterVector<>::type pixel(ClusterVector<>::N);
-    
-    #pragma omp for schedule(dynamic)
-    for (i = 0; i < (int)reps; i++) {
-      clusterer = new KMeansClusterer<>(k, seed+i);
+
+#pragma omp for schedule(dynamic)
+    for (i = 0; i < (int) reps; i++) {
+      clusterer = new KMeansClusterer<>(k, seed + i);
       clusterer->cluster(data, MAX_ITERS);
       error = clusterer->getError();
-      #pragma omp critical
+#pragma omp critical
       {
         if (error < minError) {
           delete minClusterer;
@@ -581,25 +543,22 @@ void ClusterModel::cluster() {
         //wxYield();
       }
     }
-    assert (minClusterer != NULL);
+    assert(minClusterer != NULL);
 
     /* cluster colours */
     ColourSpace cs;
     ColourSpace::rgb_t rgb(3);
     ColourSpace::hsl_t hsl(3);
-    #pragma omp for
-    for (i = 0; i < (int)k; i++) {
-      pixel = 127.0*minClusterer->getCentroid(i) +
-          ublas::scalar_vector<float>(3,128.0);
-      rgb(0) = static_cast<unsigned char>(pixel(0));
-      rgb(1) = static_cast<unsigned char>(pixel(1));
-      rgb(2) = static_cast<unsigned char>(pixel(2));
+#pragma omp for
+    for (i = 0; i < (int) k; i++) {
+      pixel = 0.5f * minClusterer->getCentroid(i)
+          + ublas::scalar_vector<float>(3, 0.5f);
+      rgb = pixel;
       cs.rgb2hsl(rgb, hsl);
       hsl(1) = 0.8f;
-      hsl(2) = 190.0f;
+      hsl(2) = 0.8f;
       cs.hsl2rgb(hsl, rgb);
-
-      colours[i].Set(rgb(0), rgb(1), rgb(2));
+      colours[i].Set(uround(rgb(0)), uround(rgb(1)), uround(rgb(2)));
     }
   }
 }
